//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "rtklib.h"
#include "mondlg.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TMonitorDialog *MonitorDialog;

#define SQRT(x)     ((x)<0.0?sqrt(-(x)):sqrt(x))

extern rtksvr_t rtksvr;		// rtk server struct

//---------------------------------------------------------------------------
__fastcall TMonitorDialog::TMonitorDialog(TComponent* Owner)
	: TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDialog::FormShow(TObject *Sender)
{
	TypeF=0;
	Timer->Enabled=true;
	ClearTable();
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDialog::FormHide(TObject *Sender)
{
	Timer->Enabled=false;
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDialog::BtnCloseClick(TObject *Sender)
{
	Close();
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDialog::TypeChange(TObject *Sender)
{
	TypeF=Type->ItemIndex;
	ClearTable();	
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDialog::TimerTimer(TObject *Sender)
{
	switch (TypeF) {
		case 0: ShowRtk(); break;
		case 1: ShowSat(); break;
		case 2: ShowEst(); break;
		case 3: ShowObs(); break;
		case 4: ShowNav(); break;
		case 5: ShowStr(); break;
		case 6: ShowSbsMsg(); break;
		case 7: ShowSbsLong(); break;
		case 8: ShowSbsIono(); break;
		case 9: ShowSbsFast(); break;
		case 10: ShowSbsNav(); break;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDialog::ClearTable(void)
{
	switch (TypeF) {
		case 0: SetRtk(); break;
		case 1: SetSat(); break;
		case 2: SetEst(); break;
		case 3: SetObs(); break;
		case 4: SetNav(); break;
		case 5: SetStr(); break;
		case 6: SetSbsMsg(); break;
		case 7: SetSbsLong(); break;
		case 8: SetSbsIono(); break;
		case 9: SetSbsFast(); break;
		case 10: SetSbsNav(); break;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDialog::SetRtk(void)
{
	AnsiString label[]={"Parameter","Value"};
	int width[]={220,380};
	
	Tbl->ColCount=2;
	Tbl->RowCount=2;
	for (int i=0;i<Tbl->ColCount;i++) {
		Tbl->ColWidths[i]=width[i];
		Tbl->Cells[i][0]=label[i];
		Tbl->Cells[i][1]="";
	}
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDialog::ShowRtk(void)
{
	rtksvr_t svr;
	AnsiString s;
	AnsiString svrstate[]={"Stop","Run"};
	AnsiString sol[]={"-","Fix","Float","SBAS","DGPS","Single"};
	AnsiString mode[]={"Single","DGPS","Kinematic","Static","Moving-Base"};
	AnsiString freq[]={"-","L1","L1+L2","L1+L2+L5"};
	int i=1,j,k;
	char tstr[64],*ant;
	double *del,*off1,*off2,del0[3]={0},runtime,rt[3]={0},dop[4]={0},azel[MAXSAT*2];
	
	rtksvrlock(&rtksvr);
	svr=rtksvr;
	rtksvrunlock(&rtksvr);
	if (svr.state) {
		runtime=(double)(tickget()-svr.tick)/1000.0;
		rt[0]=floor(runtime/3600.0); runtime-=rt[0]*3600.0;
		rt[1]=floor(runtime/60.0); rt[2]=runtime-rt[1]*60.0;
	}
	for (j=k=0;j<MAXSAT;j++) {
		if (svr.rtk.opt.mode==PMODE_SINGLE&&!svr.rtk.vs[0][j]) continue;
		if (svr.rtk.opt.mode!=PMODE_SINGLE&&!svr.rtk.vsat[j][0]) continue;
		azel[k*2]=svr.rtk.az[0][j]; azel[1+k*2]=svr.rtk.el[0][j];
		k++;
	}
	dops(k,azel,0.0,dop);
	
	Label->Caption="";
	Tbl->RowCount=59;
	Tbl->Cells[0][i]="RTKLIB Version";  Tbl->Cells[1][i++]=VER_RTKLIB;
	Tbl->Cells[0][i]="RTK Server Thread"; Tbl->Cells[1][i++]=s.sprintf("%d",(int)(svr.thread));
	Tbl->Cells[0][i]="RTK Server State"; Tbl->Cells[1][i++]=svrstate[svr.state];
	Tbl->Cells[0][i]="Processing Cycle (ms)"; Tbl->Cells[1][i++]=s.sprintf("%d",svr.cycle);
	Tbl->Cells[0][i]="Input Buffer Size (bytes)"; Tbl->Cells[1][i++]=s.sprintf("%d",svr.buffsize);
	Tbl->Cells[0][i]="Positioning Mode"; Tbl->Cells[1][i++]=mode[svr.rtk.opt.mode];
	Tbl->Cells[0][i]="Frequencies"; Tbl->Cells[1][i++]=freq[svr.rtk.opt.nf];
	Tbl->Cells[0][i]="Elevation Mask (deg)"; Tbl->Cells[1][i++]=s.sprintf("%.0f",svr.rtk.opt.elmin*R2D);
	Tbl->Cells[0][i]="SNR Mask (dbHz)"; Tbl->Cells[1][i++]=s.sprintf("%.0f",svr.rtk.opt.snrmin);
	Tbl->Cells[0][i]="SBAS Corrections";
	Tbl->Cells[1][i++]=s.sprintf("%s,%s,%s,%s",
				svr.rtk.opt.sbascorr&SBSOPT_LCORR?"On":"Off",
				svr.rtk.opt.sbascorr&SBSOPT_ICORR?"On":"Off",
				svr.rtk.opt.sbascorr&SBSOPT_FCORR?"On":"Off",
				svr.rtk.opt.sbascorr&SBSOPT_RANGE?"On":"Off");
	Tbl->Cells[0][i]="Data in Input Buffer (bytes)"; Tbl->Cells[1][i++]=s.sprintf("%d,%d",svr.nb[0],svr.nb[1]);
	Tbl->Cells[0][i]="Accumulated Time to run"; Tbl->Cells[1][i++]=s.sprintf("%02.0f:%02.0f:%04.1f",rt[0],rt[1],rt[2]);
	Tbl->Cells[0][i]="# of Input Data Rover";
	Tbl->Cells[1][i++]=s.sprintf("%d,%d,%d,%d,%d,%d",
								 svr.nmsg[0][0],svr.nmsg[0][1],svr.nmsg[0][2],
								 svr.nmsg[0][3],svr.nmsg[0][8],svr.nmsg[0][9]);
	Tbl->Cells[0][i]="# of Input Data Base Station";
	Tbl->Cells[1][i++]=s.sprintf("%d,%d,%d,%d,%d,%d",
								 svr.nmsg[1][0],svr.nmsg[1][1],svr.nmsg[1][2],
								 svr.nmsg[1][3],svr.nmsg[1][8],svr.nmsg[1][9]);
	Tbl->Cells[0][i]="Solution Status"; Tbl->Cells[1][i++]=sol[svr.rtkstat];
	time2str(svr.rtk.time[0],tstr,9);
	Tbl->Cells[0][i] ="Time of Receiver Clock Rover";
	Tbl->Cells[1][i++]=svr.rtk.time[0].time?tstr:"-";
	Tbl->Cells[0][i] ="Time of Receiver Clock Base Station";
	time2str(svr.rtk.time[1],tstr,9);
	Tbl->Cells[1][i++]=svr.rtk.time[1].time?tstr:"-";
	Tbl->Cells[0][i]="Solution Interval (s)"; Tbl->Cells[1][i++]=s.sprintf("%.3f",svr.rtk.tt);
	Tbl->Cells[0][i]="Time of Receiver Clock Base Station";
	Tbl->Cells[0][i]="Age of Differential (s)"; Tbl->Cells[1][i++] =s.sprintf("%.3f",svr.rtk.age);
	Tbl->Cells[0][i]="Ratio for AR Validation"; Tbl->Cells[1][i++]=s.sprintf("%.3f",svr.rtk.ambratio);
	Tbl->Cells[0][i]="# of Satellites Rover"; Tbl->Cells[1][i++]=s.sprintf("%d",svr.obs[0].n);
	Tbl->Cells[0][i]="# of Satellites Base Station"; Tbl->Cells[1][i++]=s.sprintf("%d",svr.obs[1].n);
	Tbl->Cells[0][i]="# of Valid Satellites"; Tbl->Cells[1][i++]=s.sprintf("%d",svr.rtk.ns);
	Tbl->Cells[0][i]="GDOP/PDOP/HDOP/VDOP"; Tbl->Cells[1][i++]=s.sprintf("%.1f,%.1f,%.1f,%.1f",dop[0],dop[1],dop[2],dop[3]);
	Tbl->Cells[0][i]="# of Real Estimated States"; Tbl->Cells[1][i++]=s.sprintf("%d",svr.rtk.na);
	Tbl->Cells[0][i]="# of All Estimated States"; Tbl->Cells[1][i++]=s.sprintf("%d",svr.rtk.nx);
	Tbl->Cells[0][i]="Pos X Single (m) Rover"; Tbl->Cells[1][i++]=s.sprintf("%.3f",svr.rtk.rr[0][0]);
	Tbl->Cells[0][i]="Pos Y Single (m) Rover"; Tbl->Cells[1][i++]=s.sprintf("%.3f",svr.rtk.rr[0][1]);
	Tbl->Cells[0][i]="Pos Z Single (m) Rover"; Tbl->Cells[1][i++]=s.sprintf("%.3f",svr.rtk.rr[0][2]);
	Tbl->Cells[0][i]="Vel X Single (m) Rover"; Tbl->Cells[1][i++]=s.sprintf("%.3f",svr.rtk.rr[0][3]);
	Tbl->Cells[0][i]="Vel Y Single (m) Rover"; Tbl->Cells[1][i++]=s.sprintf("%.3f",svr.rtk.rr[0][4]);
	Tbl->Cells[0][i]="Vel Z Single (m) Rover"; Tbl->Cells[1][i++]=s.sprintf("%.3f",svr.rtk.rr[0][5]);
	Tbl->Cells[0][i]="Pos X Float (m) Rover"; Tbl->Cells[1][i++]=s.sprintf("%.3f",svr.rtk.x?svr.rtk.x[0]:0);
	Tbl->Cells[0][i]="Pos Y Float (m) Rover"; Tbl->Cells[1][i++]=s.sprintf("%.3f",svr.rtk.x?svr.rtk.x[1]:0);
	Tbl->Cells[0][i]="Pos Z Float (m) Rover"; Tbl->Cells[1][i++]=s.sprintf("%.3f",svr.rtk.x?svr.rtk.x[2]:0);
	Tbl->Cells[0][i]="Pos X Float Std (m) Rover"; Tbl->Cells[1][i++]=s.sprintf("%.3f",svr.rtk.P?SQRT(svr.rtk.P[0]):0);
	Tbl->Cells[0][i]="Pos Y Float Std (m) Rover"; Tbl->Cells[1][i++]=s.sprintf("%.3f",svr.rtk.P?SQRT(svr.rtk.P[1+1*svr.rtk.nx]):0);
	Tbl->Cells[0][i]="Pos Z Float Std (m) Rover"; Tbl->Cells[1][i++]=s.sprintf("%.3f",svr.rtk.P?SQRT(svr.rtk.P[2+2*svr.rtk.nx]):0);
	Tbl->Cells[0][i]="Pos X Fixed (m) Rover"; Tbl->Cells[1][i++]=s.sprintf("%.3f",svr.rtk.xa?svr.rtk.xa[0]:0);
	Tbl->Cells[0][i]="Pos Y Fixed (m) Rover"; Tbl->Cells[1][i++]=s.sprintf("%.3f",svr.rtk.xa?svr.rtk.xa[1]:0);
	Tbl->Cells[0][i]="Pos Z Fixed (m) Rover"; Tbl->Cells[1][i++]=s.sprintf("%.3f",svr.rtk.xa?svr.rtk.xa[2]:0);
	Tbl->Cells[0][i]="Pos X Fixed Std (m) Rover"; Tbl->Cells[1][i++]=s.sprintf("%.3f",svr.rtk.Pa?SQRT(svr.rtk.Pa[0]):0);
	Tbl->Cells[0][i]="Pos Y Fixed Std (m) Rover"; Tbl->Cells[1][i++]=s.sprintf("%.3f",svr.rtk.Pa?SQRT(svr.rtk.Pa[1+1*svr.rtk.na]):0);
	Tbl->Cells[0][i]="Pos Z Fixed Std (m) Rover"; Tbl->Cells[1][i++]=s.sprintf("%.3f",svr.rtk.Pa?SQRT(svr.rtk.Pa[2+2*svr.rtk.na]):0);
	Tbl->Cells[0][i]="Pos X (m) Base Station"; Tbl->Cells[1][i++]=s.sprintf("%.3f",svr.rtk.rr[1][0]);
	Tbl->Cells[0][i]="Pos Y (m) Base Station"; Tbl->Cells[1][i++]=s.sprintf("%.3f",svr.rtk.rr[1][1]);
	Tbl->Cells[0][i]="Pos Z (m) Base Station"; Tbl->Cells[1][i++]=s.sprintf("%.3f",svr.rtk.rr[1][2]);
	Tbl->Cells[0][i]="Vel X (m) Base Station"; Tbl->Cells[1][i++]=s.sprintf("%.3f",svr.rtk.rr[1][3]);
	Tbl->Cells[0][i]="Vel Y (m) Base Station"; Tbl->Cells[1][i++]=s.sprintf("%.3f",svr.rtk.rr[1][4]);
	Tbl->Cells[0][i]="Vel Z (m) Base Station"; Tbl->Cells[1][i++]=s.sprintf("%.3f",svr.rtk.rr[1][5]);
	ant=svr.rtk.opt.pcvu?svr.rtk.opt.pcvu->ant:"";
	off1=svr.rtk.opt.pcvu?svr.rtk.opt.pcvu->off[0]:del0;
	off2=svr.rtk.opt.pcvu?svr.rtk.opt.pcvu->off[1]:del0;
	del=svr.rtk.opt.pcvu?svr.rtk.opt.pcvu->ecc:del0;
	Tbl->Cells[0][i]="Antenna Type Rover"; Tbl->Cells[1][i++]=ant;
	Tbl->Cells[0][i]="Ant Phase Center L1 E/N/U (m) Rover"; Tbl->Cells[1][i++]=s.sprintf("%.3f,%.3f,%.3f",off1[0],off1[1],off1[2]);
	Tbl->Cells[0][i]="Ant Phase Center L2 E/N/U (m) Rover"; Tbl->Cells[1][i++]=s.sprintf("%.3f,%.3f,%.3f",off2[0],off2[1],off2[2]);
	Tbl->Cells[0][i]="Ant Delta E/N/U (m) Rover"; Tbl->Cells[1][i++]=s.sprintf("%.3f,%.3f,%.3f",del[0],del[1],del[2]);
	ant=svr.rtk.opt.pcvr?svr.rtk.opt.pcvr->ant:"";
	off1=svr.rtk.opt.pcvr?svr.rtk.opt.pcvr->off[0]:del0;
	off2=svr.rtk.opt.pcvr?svr.rtk.opt.pcvr->off[1]:del0;
	del=svr.rtk.opt.pcvr?svr.rtk.opt.pcvr->ecc:del0;
	Tbl->Cells[0][i]="Antenna Type Base Station"; Tbl->Cells[1][i++]=ant;
	Tbl->Cells[0][i]="Ant Phase Center L1 E/N/U (m) Base Station"; Tbl->Cells[1][i++]=s.sprintf("%.3f,%.3f,%.3f",off1[0],off1[1],off1[2]);
	Tbl->Cells[0][i]="Ant Phase Center L2 E/N/U (m) Base Station"; Tbl->Cells[1][i++]=s.sprintf("%.3f,%.3f,%.3f",off2[0],off2[1],off2[2]);
	Tbl->Cells[0][i]="Ant Delta E/N/U (m) Base Station"; Tbl->Cells[1][i++]=s.sprintf("%.3f,%.3f,%.3f",del[0],del[1],del[2]);
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDialog::SetSat(void)
{
	AnsiString label[]={
		"PRN","C1","L1","L2","Slip1","Slip2","Lock1","Lock2","Out1","Out2",
		"L1 Residual (m)","L2 Residual (m)","Azimuth (deg)","Elevation (deg)",
		"LG (m)"
	};
	int width[]={25,30,30,30,30,30,45,45,45,45,45,45,45,45,60};
	
	Tbl->ColCount=15;
	Tbl->RowCount=2;
	for (int i=0;i<Tbl->ColCount;i++) {
		Tbl->ColWidths[i]=width[i];
		Tbl->Cells[i][0]=label[i];
		Tbl->Cells[i][1]="";
	}
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDialog::ShowSat(void)
{
	rtksvr_t svr;
	AnsiString s;
	
	rtksvrlock(&rtksvr);
	svr=rtksvr;
	rtksvrunlock(&rtksvr);
	Tbl->RowCount=MAXSAT+1;
	Label->Caption="";
	for (int i=0;i<MAXSAT;i++) {
		int j=0;
		Tbl->Cells[j++][i+1]=s.sprintf("%d",PRN(i+1));
		Tbl->Cells[j++][i+1]=svr.rtk.vs[0][i]?"OK":"-";
		Tbl->Cells[j++][i+1]=svr.rtk.vsat[i][0]?"OK":"-";
		Tbl->Cells[j++][i+1]=svr.rtk.vsat[i][1]?"OK":"-";
		Tbl->Cells[j++][i+1]=svr.rtk.slip[i][0]?"S":"-";
		Tbl->Cells[j++][i+1]=svr.rtk.slip[i][1]?"S":"-";
		Tbl->Cells[j++][i+1]=s.sprintf("%d",svr.rtk.lock[i][0]);
		Tbl->Cells[j++][i+1]=s.sprintf("%d",svr.rtk.lock[i][1]);
		Tbl->Cells[j++][i+1]=s.sprintf("%d",svr.rtk.outc[i][0]);
		Tbl->Cells[j++][i+1]=s.sprintf("%d",svr.rtk.outc[i][1]);
		Tbl->Cells[j++][i+1]=s.sprintf("%.4f",svr.rtk.res[i][0]);
		Tbl->Cells[j++][i+1]=s.sprintf("%.4f",svr.rtk.res[i][1]);
		Tbl->Cells[j++][i+1]=s.sprintf("%.1f",svr.rtk.az[0][i]*R2D);
		Tbl->Cells[j++][i+1]=s.sprintf("%.1f",svr.rtk.el[0][i]*R2D);
		Tbl->Cells[j++][i+1]=s.sprintf("%.3f",svr.rtk.gf[i]);
	}
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDialog::SetEst(void)
{
	AnsiString label[]={
		"State","Estimate Float","Std Float","Estimate Fixed","Std Fixed"
	};
	int width[]={40,100,100,100,100};
	
	Tbl->ColCount=5;
	Tbl->RowCount=2;
	for (int i=0;i<Tbl->ColCount;i++) {
		Tbl->ColWidths[i]=width[i];
		Tbl->Cells[i][0]=label[i];
		Tbl->Cells[i][1]="";
	}
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDialog::ShowEst(void)
{
	rtksvr_t svr;
	AnsiString s,s0="-";
	char tstr[64];

	rtksvrlock(&rtksvr);
	svr=rtksvr;
	rtksvrunlock(&rtksvr);
	Tbl->RowCount=svr.rtk.nx+1>2?svr.rtk.nx+1:2;
	time2str(svr.rtk.time[0],tstr,9);
	Label->Caption=svr.rtk.time[0].time?s.sprintf("Time: %s",tstr):s0;
	for (int i=0;i<svr.rtk.nx;i++) {
		int j=0;
		Tbl->Cells[j++][i+1]=s.sprintf("X_%d",i+1);
		Tbl->Cells[j++][i+1]=s.sprintf("%.3f",svr.rtk.x[i]);
		Tbl->Cells[j++][i+1]=s.sprintf("%.3f",SQRT(svr.rtk.P[i+i*svr.rtk.nx]));
		Tbl->Cells[j++][i+1]=s.sprintf("%.3f",svr.rtk.na<i?0:svr.rtk.xa[i]);
		Tbl->Cells[j++][i+1]=s.sprintf("%.3f",svr.rtk.na<i?0:SQRT(svr.rtk.Pa[i+i*svr.rtk.na]));
	}
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDialog::SetObs(void)
{
	AnsiString label[]={
		"Trcv","PRN","RCV","L1 (cycle)","L2 (cycle)","C1 (m)","P2 (m)",
		"D1 (Hz)","D2 (Hz)","S1 (dBHz)","S2 (dBHz)","LLI1","LLI2"
	};
	int width[]={135,25,25,85,85,80,80,60,60,30,30,30,30};
	
	Tbl->ColCount=13;
	Tbl->RowCount=2;
	for (int i=0;i<Tbl->ColCount;i++) {
		Tbl->ColWidths[i]=width[i];
		Tbl->Cells[i][0]=label[i];
		Tbl->Cells[i][1]="";
	}
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDialog::ShowObs(void)
{
	AnsiString s;
	obsd_t obs[MAXOBS*2];
	char tstr[64];
	int n=0;
	
	rtksvrlock(&rtksvr);
	for (int i=0;i<rtksvr.obs[0].n&&n<MAXOBS*2;i++) {
		obs[n++]=rtksvr.obs[0].data[i];
	}
	for (int i=0;i<rtksvr.obs[1].n&&n<MAXOBS*2;i++) {
		obs[n++]=rtksvr.obs[1].data[i];
	}
	rtksvrunlock(&rtksvr);
	
	Tbl->RowCount=n+1<2?2:n+1;
	Label->Caption="";
	for (int i=0;i<n;i++) {
		int j=0;
		time2str(obs[i].time,tstr,3);
		Tbl->Cells[j++][i+1]=tstr;
		Tbl->Cells[j++][i+1]=s.sprintf("%d",obs[i].sat);
		Tbl->Cells[j++][i+1]=s.sprintf("%d",obs[i].rcv);
		Tbl->Cells[j++][i+1]=s.sprintf("%.3f",obs[i].L[0]);
		Tbl->Cells[j++][i+1]=s.sprintf("%.3f",obs[i].L[1]);
		Tbl->Cells[j++][i+1]=s.sprintf("%.3f",obs[i].P[0]);
		Tbl->Cells[j++][i+1]=s.sprintf("%.3f",obs[i].P[1]);
		Tbl->Cells[j++][i+1]=s.sprintf("%.3f",obs[i].D[0]);
		Tbl->Cells[j++][i+1]=s.sprintf("%.3f",obs[i].D[1]);
		Tbl->Cells[j++][i+1]=s.sprintf("%d",obs[i].SNR[0]);
		Tbl->Cells[j++][i+1]=s.sprintf("%d",obs[i].SNR[1]);
		Tbl->Cells[j++][i+1]=s.sprintf("%d",obs[i].LLI[0]);
		Tbl->Cells[j++][i+1]=s.sprintf("%d",obs[i].LLI[1]);
	}
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDialog::SetNav(void)
{
	AnsiString label[]={
		"PRN","Status","IODE","IODC","Accuracy","Health","Toe","Toc","Ttrans",
		"A (m)","e","i0 (deg)","OMEGA0 (deg)","omega (deg)","M0 (deg)",
		"deltan (deg/s)","OMEGAdot (deg/s)","IDOT (deg/s)",
		"af0 (s)","af1 (s/s)","af2 (s/s2)","TGD (s)"
	};
	int width[]={25,30,30,30,25,25,115,115,115,80,70,60,60,60,60,70,70,70,70,70,70,70};
	
	Tbl->ColCount=21;
	Tbl->RowCount=2;
	for (int i=0;i<Tbl->ColCount;i++) {
		Tbl->ColWidths[i]=width[i];
		Tbl->Cells[i][0]=label[i];
		Tbl->Cells[i][1]="";
	}
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDialog::ShowNav(void)
{
	rtksvr_t svr;
	AnsiString s;
	double *ion,*utc;
	char tstr[64];
	
	rtksvrlock(&rtksvr);
	svr=rtksvr;
	rtksvrunlock(&rtksvr);
	Tbl->RowCount=MAXSAT+1;
	ion=svr.nav.ion;
	utc=svr.nav.utc;
	Label->Caption=s.sprintf("ION/UTC: %.2E %.2E %.2E %.2E %.2E %.2E %.2E %.2E %.2E %.2E %.2E %.2E",
							 ion[0],ion[1],ion[2],ion[3],
							 ion[4],ion[5],ion[6],ion[7],
							 utc[0],utc[1],utc[2],utc[3]);
	for (int i=0;i<MAXSAT;i++) {
		int j=0,valid;
		valid=svr.nav.eph[i].toe.time!=0&&!svr.nav.eph[i].svh&&
			  fabs(timediff(svr.rtk.time[0],svr.nav.eph[i].toe))<=MAXDTOE;
		Tbl->Cells[j++][i+1]=s.sprintf("%d",PRN(i+1));
		Tbl->Cells[j++][i+1]=valid?"OK":"-";
		Tbl->Cells[j++][i+1]=s.sprintf("%d",svr.nav.eph[i].iode);
		Tbl->Cells[j++][i+1]=s.sprintf("%d",svr.nav.eph[i].iodc);
		Tbl->Cells[j++][i+1]=s.sprintf("%d",svr.nav.eph[i].sva);
		Tbl->Cells[j++][i+1]=s.sprintf("%02x",svr.nav.eph[i].svh);
		if (svr.nav.eph[i].toe.time!=0) time2str(svr.nav.eph[i].toe,tstr,0); else strcpy(tstr,"-");
		Tbl->Cells[j++][i+1]=s.sprintf("%s",tstr);
		if (svr.nav.eph[i].toc.time!=0) time2str(svr.nav.eph[i].toc,tstr,0); else strcpy(tstr,"-");
		Tbl->Cells[j++][i+1]=s.sprintf("%s",tstr);
		if (svr.nav.eph[i].ttr.time!=0) time2str(svr.nav.eph[i].ttr,tstr,0); else strcpy(tstr,"-");
		Tbl->Cells[j++][i+1]=s.sprintf("%s",tstr);
		Tbl->Cells[j++][i+1]=s.sprintf("%.3f",svr.nav.eph[i].A);
		Tbl->Cells[j++][i+1]=s.sprintf("%.8f",svr.nav.eph[i].e);
		Tbl->Cells[j++][i+1]=s.sprintf("%.5f",svr.nav.eph[i].i0*R2D);
		Tbl->Cells[j++][i+1]=s.sprintf("%.5f",svr.nav.eph[i].OMG0*R2D);
		Tbl->Cells[j++][i+1]=s.sprintf("%.5f",svr.nav.eph[i].omg*R2D);
		Tbl->Cells[j++][i+1]=s.sprintf("%.5f",svr.nav.eph[i].M0*R2D);
		Tbl->Cells[j++][i+1]=s.sprintf("%.4E",svr.nav.eph[i].deln*R2D);
		Tbl->Cells[j++][i+1]=s.sprintf("%.4E",svr.nav.eph[i].OMGd*R2D);
		Tbl->Cells[j++][i+1]=s.sprintf("%.4E",svr.nav.eph[i].idot*R2D);
		Tbl->Cells[j++][i+1]=s.sprintf("%.4E",svr.nav.eph[i].f0);
		Tbl->Cells[j++][i+1]=s.sprintf("%.4E",svr.nav.eph[i].f1);
		Tbl->Cells[j++][i+1]=s.sprintf("%.4E",svr.nav.eph[i].f2);
		Tbl->Cells[j++][i+1]=s.sprintf("%.4E",svr.nav.eph[i].tgd);
	}
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDialog::SetStr(void)
{
	AnsiString label[]={
		"Stream","Type","Mode","State","Input (bytes)","Input (bps)",
		"Output (bytes)","Output (bps)","Path","Message"
	};
	int width[]={95,70,35,35,70,70,70,70,220,220};
	
	Tbl->ColCount=10;
	Tbl->RowCount=2;
	for (int i=0;i<Tbl->ColCount;i++) {
		Tbl->ColWidths[i]=width[i];
		Tbl->Cells[i][0]=label[i];
		Tbl->Cells[i][1]="";
	}
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDialog::ShowStr(void)
{
	AnsiString ch[]={
		"Input Rover","Input Base Station","Output Solution 1",
		"Output Solution 2","Log Rover","Log Base Station"
	};
	AnsiString type[]={
		"-","Serial","File","TCP Server","TCP Client","UDP","NTRIP Server",
		"NTRIP Client"
	};
	AnsiString state[]={"Error","Close","Open"};
	AnsiString s,mode;
	rtksvr_t svr;
	
	rtksvrlock(&rtksvr);
	svr=rtksvr;
	rtksvrunlock(&rtksvr);
	Tbl->RowCount=MAXSTRRTK+1;
	Label->Caption="";
	for (int i=0;i<MAXSTRRTK;i++) {
		int j=0;
		Tbl->Cells[j++][i+1]=ch[i];
		Tbl->Cells[j++][i+1]=type[svr.stream[i].type];
		if (svr.stream[i].mode&STR_MODE_R) mode="R"; else mode="";
		if (svr.stream[i].mode&STR_MODE_W) mode=mode+(mode==""?"":"/")+"W";
		if (svr.stream[i].mode&STR_MODE_T) mode=mode+(mode==""?"":"/")+"T";
		Tbl->Cells[j++][i+1]=mode;
		Tbl->Cells[j++][i+1]=state[svr.stream[i].state+1];
		Tbl->Cells[j++][i+1]=s.sprintf("%d",svr.stream[i].inb);
		Tbl->Cells[j++][i+1]=s.sprintf("%d",svr.stream[i].inr);
		Tbl->Cells[j++][i+1]=s.sprintf("%d",svr.stream[i].outb);
		Tbl->Cells[j++][i+1]=s.sprintf("%d",svr.stream[i].outr);
		Tbl->Cells[j++][i+1]=svr.stream[i].path;
		Tbl->Cells[j++][i+1]=svr.stream[i].msg;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDialog::SetSbsMsg(void)
{
	AnsiString label[]={
		"Trcv","PRN","Type","Message","Contents"
	};
	int width[]={115,25,25,420,200};
	
	Tbl->ColCount=5;
	Tbl->RowCount=2;
	for (int i=0;i<Tbl->ColCount;i++) {
		Tbl->ColWidths[i]=width[i];
		Tbl->Cells[i][0]=label[i];
		Tbl->Cells[i][1]="";
	}
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDialog::ShowSbsMsg(void)
{
	AnsiString s;
	rtksvr_t svr;
	const char *content[]={
		"For Testing","PRN Mask","Fast Corrections","Fast Corrections",
		"Fast Corrections","Fast Corrections","Integrity Information",
		"Fast Correction Degradation Factor","GEO Navigation Message",
		"Degradation Parameters","WAAS Network Time/UTC Offset Parameters",
		"GEO Satellite Almanacs","Ionospheric Grid Point Masks",
		"Mixed Fast Corrections/Long Term Satellite Error Corrections",
		"Long Term Satellite Error Corrections","Ionospheric Delay Corrections",
		"WAAS Service Messages","Clock-Ephemeris Covariance Matrix Message",
		"Internal Test Message","Null Message",""
	};
	const int id[]={0,1,2,3,4,5,6,7,9,10,12,17,18,24,25,26,27,28,62,63,-1};
	char str[256],*p;
	
	rtksvrlock(&rtksvr);
	svr=rtksvr;
	rtksvrunlock(&rtksvr);
	Tbl->RowCount=svr.nsbs<=0?2:svr.nsbs+1;
	Label->Caption="";
	for (int i=0;i<svr.nsbs;i++) {
		int j=0,k,type;
		time2str(gpst2time(svr.sbsmsg[i].week,svr.sbsmsg[i].tow),str,0);
		Tbl->Cells[j++][i+1]=str;
		Tbl->Cells[j++][i+1]=s.sprintf("%d",svr.sbsmsg[i].sat);
		type=svr.sbsmsg[i].msg[1]>>2;
		Tbl->Cells[j++][i+1]=s.sprintf("%d",type);
		p=str;
		for (k=0;k<29;k++) p+=sprintf(p,"%02X",svr.sbsmsg[i].msg[k]);
		Tbl->Cells[j++][i+1]=str;
		for (k=0;id[k]>=0;k++) if (type==id[k]) break;
		Tbl->Cells[j++][i+1]=id[k]<0?"?":content[k];
	}
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDialog::SetSbsLong(void)
{
	AnsiString label[]={
		"PRN","Status","IODE","dX (m)","dY (m)","dZ (m)","dVX (m/s)",
		"dVY (m/s)","dVZ (m/s)","daf0 (ns)","daf1 (ns/s)","T0"
	};
	int width[]={25,30,30,55,55,55,55,55,55,55,55,115};
	
	Tbl->ColCount=12;
	Tbl->RowCount=2;
	for (int i=0;i<Tbl->ColCount;i++) {
		Tbl->ColWidths[i]=width[i];
		Tbl->Cells[i][0]=label[i];
		Tbl->Cells[i][1]="";
	}
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDialog::ShowSbsLong(void)
{
	AnsiString s;
	rtksvr_t svr;
	sbsstat_t *stat;
	sbssatp_t *satp;
	char tstr[64];
	
	rtksvrlock(&rtksvr);
	svr=rtksvr;
	rtksvrunlock(&rtksvr);
	stat=&svr.sbsstat;
	Label->Caption="";
	Tbl->RowCount=stat->sbssat.nprn<=0?2:stat->sbssat.nprn+1;
	Label->Caption=s.sprintf("IODP:%2d  System Latency:%2d s",
							 stat->sbssat.iodp,stat->sbssat.tlat);
	for (int i=0;i<Tbl->RowCount;i++) {
		int j=0,valid;
		satp=stat->sbssat.sat+i;
		valid=timediff(svr.rtk.time[0],satp->lcorr.t0)<=MAXSBSAGEL&&
			  satp->lcorr.t0.time;
		Tbl->Cells[j++][i+1]=s.sprintf("%d",satp->prn);
		Tbl->Cells[j++][i+1]=valid?"OK":"-";
		Tbl->Cells[j++][i+1]=s.sprintf("%d",satp->lcorr.iode);
		Tbl->Cells[j++][i+1]=s.sprintf("%.3f",satp->lcorr.dpos[0]);
		Tbl->Cells[j++][i+1]=s.sprintf("%.3f",satp->lcorr.dpos[1]);
		Tbl->Cells[j++][i+1]=s.sprintf("%.3f",satp->lcorr.dpos[2]);
		Tbl->Cells[j++][i+1]=s.sprintf("%.4f",satp->lcorr.dvel[0]);
		Tbl->Cells[j++][i+1]=s.sprintf("%.4f",satp->lcorr.dvel[1]);
		Tbl->Cells[j++][i+1]=s.sprintf("%.4f",satp->lcorr.dvel[2]);
		Tbl->Cells[j++][i+1]=s.sprintf("%.3f",satp->lcorr.daf0*1E9);
		Tbl->Cells[j++][i+1]=s.sprintf("%.4f",satp->lcorr.daf1*1E9);
		if (satp->lcorr.t0.time) time2str(satp->lcorr.t0,tstr,0);
		else strcpy(tstr,"-");
		Tbl->Cells[j++][i+1]=tstr;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDialog::SetSbsIono(void)
{
	AnsiString label[]={
		"IODI","Lat (deg)","Lon (deg)","GIVEI","Delay (m)","T0"
	};
	int width[]={30,50,50,30,60,115};
	
	Tbl->ColCount=6;
	Tbl->RowCount=2;
	for (int i=0;i<Tbl->ColCount;i++) {
		Tbl->ColWidths[i]=width[i];
		Tbl->Cells[i][0]=label[i];
		Tbl->Cells[i][1]="";
	}
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDialog::ShowSbsIono(void)
{
	AnsiString s,s0="-";
	rtksvr_t svr;
	sbsion_t *ion;
	char tstr[64];
	int n=0;
	
	rtksvrlock(&rtksvr);
	svr=rtksvr;
	rtksvrunlock(&rtksvr);
	Label->Caption="";
	for (int i=0;i<MAXBAND+1;i++) {
		ion=svr.sbsstat.sbsion+i;
		for (int j=0;j<ion->nigp;j++) {
			int k=0;
			Tbl->Cells[k++][n+1]=s.sprintf("%d",ion->iodi);
			Tbl->Cells[k++][n+1]=s.sprintf("%d",ion->igp[j].lat);
			Tbl->Cells[k++][n+1]=s.sprintf("%d",ion->igp[j].lon);
			Tbl->Cells[k++][n+1]=ion->igp[j].give?s.sprintf("%d",ion->igp[j].give-1):s0;
			Tbl->Cells[k++][n+1]=s.sprintf("%.3f",ion->igp[j].delay);
			if (ion->igp[j].t0.time) time2str(ion->igp[j].t0,tstr,0);
			else strcpy(tstr,"-");
			Tbl->Cells[k++][n+1]=tstr;
			n++;
		}
	}
	Tbl->RowCount=n<=0?2:n+1;
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDialog::SetSbsFast(void)
{
	AnsiString label[]={
		"PRN","Status","PRC (m)","RRC (m)","IODF","UDREI","AI","Tof"
	};
	int width[]={25,30,60,60,30,30,30,115};
	
	Tbl->ColCount=8;
	Tbl->RowCount=2;
	for (int i=0;i<Tbl->ColCount;i++) {
		Tbl->ColWidths[i]=width[i];
		Tbl->Cells[i][0]=label[i];
		Tbl->Cells[i][1]="";
	}
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDialog::ShowSbsFast(void)
{
	AnsiString s,s0="-";
	sbsstat_t *stat;
	rtksvr_t svr;
	sbssatp_t *satp;
	char tstr[64];
	
	rtksvrlock(&rtksvr);
	svr=rtksvr;
	rtksvrunlock(&rtksvr);
	Label->Caption="";
	stat=&svr.sbsstat;
	Tbl->RowCount=stat->sbssat.nprn<=0?2:stat->sbssat.nprn+1;
	Label->Caption=s.sprintf("IODP:%2d  System Latency:%2d s",
						     stat->sbssat.iodp,stat->sbssat.tlat);
	for (int i=0;i<Tbl->RowCount;i++) {
		int j=0,valid;
		satp=stat->sbssat.sat+i;
		valid=fabs(timediff(svr.rtk.time[0],satp->fcorr.t0)<=MAXSBSAGEF)&&
			  satp->fcorr.t0.time;
		Tbl->Cells[j++][i+1]=s.sprintf("%d",satp->prn);
		Tbl->Cells[j++][i+1]=valid?"OK":"-";
		Tbl->Cells[j++][i+1]=s.sprintf("%.3f",satp->fcorr.prc);
		Tbl->Cells[j++][i+1]=s.sprintf("%.4f",satp->fcorr.rrc);
		Tbl->Cells[j++][i+1]=s.sprintf("%d",satp->fcorr.iodf);
		Tbl->Cells[j++][i+1]=satp->fcorr.udre?s.sprintf("%d",satp->fcorr.udre-1):s0;
		Tbl->Cells[j++][i+1]=s.sprintf("%d",satp->fcorr.ai);
		if (satp->fcorr.t0.time) time2str(satp->fcorr.t0,tstr,0);
		else strcpy(tstr,"-");
		Tbl->Cells[j++][i+1]=tstr;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDialog::SetSbsNav(void)
{
	AnsiString label[]={
		"PRN","Status","T0","URA","X (m)","Y (m)","Z (m)","VX (m/s)","VY (m/s)",
		"VZ (m/s)","AX (m/s2)","AY (m/s2)","AZ (m/s2)","af0 (ns)","af1 (ns/s)"
	};
	int width[]={25,30,115,30,75,75,75,60,60,60,65,65,65,55,55};
	
	Tbl->ColCount=15;
	Tbl->RowCount=2;
	for (int i=0;i<Tbl->ColCount;i++) {
		Tbl->ColWidths[i]=width[i];
		Tbl->Cells[i][0]=label[i];
		Tbl->Cells[i][1]="";
	}
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDialog::ShowSbsNav(void)
{
	AnsiString s,s0="-";
	rtksvr_t svr;
	sbsnav_t *nav;
	int n=0;
	char tstr[64];
	
	rtksvrlock(&rtksvr);
	svr=rtksvr;
	rtksvrunlock(&rtksvr);
	Label->Caption="";
	for (int i=0;i<MAXNGEO;i++) {
		nav=svr.sbsstat.sbsnav+i;
		if (nav->sat<=0) continue;
		int j=0,valid;
		valid=fabs(timediff(svr.rtk.time[0],nav->t0)<=MAXSBSAGEN)&&
			  nav->t0.time&&nav->ura&&nav->ura<=MAXSBSURA;
		Tbl->Cells[j++][n+1]=s.sprintf("%d",nav->sat);
		Tbl->Cells[j++][n+1]=valid?"OK":"-";
		if (nav->t0.time) time2str(nav->t0,tstr,0);
		else strcpy(tstr,"-");
		Tbl->Cells[j++][n+1]=tstr;
		Tbl->Cells[j++][n+1]=nav->ura?s.sprintf("%d",nav->ura-1):s0;
		Tbl->Cells[j++][n+1]=s.sprintf("%.2f",nav->pos[0]);
		Tbl->Cells[j++][n+1]=s.sprintf("%.2f",nav->pos[1]);
		Tbl->Cells[j++][n+1]=s.sprintf("%.2f",nav->pos[2]);
		Tbl->Cells[j++][n+1]=s.sprintf("%.6f",nav->vel[0]);
		Tbl->Cells[j++][n+1]=s.sprintf("%.6f",nav->vel[1]);
		Tbl->Cells[j++][n+1]=s.sprintf("%.6f",nav->vel[2]);
		Tbl->Cells[j++][n+1]=s.sprintf("%.7f",nav->acc[0]);
		Tbl->Cells[j++][n+1]=s.sprintf("%.7f",nav->acc[1]);
		Tbl->Cells[j++][n+1]=s.sprintf("%.7f",nav->acc[2]);
		Tbl->Cells[j++][n+1]=s.sprintf("%.0f",nav->af0*1E9);
		Tbl->Cells[j++][n+1]=s.sprintf("%.1f",nav->af1*1E9);
		n++;
	}
	Tbl->RowCount=n<=0?2:n+1;
}
//---------------------------------------------------------------------------
