/*------------------------------------------------------------------------------
* str2str.c : divide a stream to multiple stream
*
*          Copyright (C) 2007-2009 by T.TAKASU, All rights reserved.
*
* version : $Revision: 1.1 $ $Date: 2008/07/17 21:54:53 $
* history : 2009/01/20  1.0 new
*-----------------------------------------------------------------------------*/
#include <signal.h>
#include <unistd.h>
#include "rtklib.h"

static const char rcsid[]="$Id:$";

static strsvr_t strsvr;			/* stream server */
static char startcmd[4096]="";
static char stopcmd[4096]="";

/* help text -----------------------------------------------------------------*/
static const char *help[]={
"",
" usage: str2str [options]",
"",
" Input data from a stream and devide and output them to multiple streams",
" The input stream can be serial, tcp client, tcp server, ntrip client, or",
" file. The output stream can be serial, tcp client, tcp server, ntrip server,",
" or file. str2str is a resident type application. To stop it, type ctr-c in",
" console if run foreground or send signal SIGINT for background process.",
" Command options are as follows.",
"",
" -in str   input stream type/path",
"             serial://port[:brate[:bsize[:parity[:stopb[:fctr]]]]]",
"             (brate=bps,bsize=7|8,parity=n|o|e,stopb=1|2,fctr=off|xon|cts)",
"             tcpsvr://:port",
"             tcpcli://addr[:port]",
"             ntrip://[user[:passwd]@]addr[:port][/mntpnt]",
"             file://path",
" -out str  output stream type/path",
"             serial://port[:brate[:bsize[:parity[:stopb[:fctr]]]]]",
"             (brate=bps,bsize=7|8,parity=n|o|e,stopb=1|2,fctr=off|xon|cts)",
"             tcpsvr://:port",
"             tcpcli://addr[:port]",
"             ntrip://[user[:passwd]@]addr[:port][/mntpnt[:str]]",
"             file://path",
" -co file  commands file at input startup [no]",
" -cc file  commands file at input shutdown [no]",
" -t file   trace file [no trace]",
" -l level  trace level [1]",
" -h        print help",
};
/* print help ----------------------------------------------------------------*/
static void printhelp(void)
{
    int i;
    for (i=0;i<sizeof(help)/sizeof(*help);i++) fprintf(stderr,"%s\n",help[i]);
    exit(0);
}
/* server stop ---------------------------------------------------------------*/
void stopsvr(int arg)
{
    strsvrstop(&strsvr,NULL);
    fprintf(stderr,"\n[str2str] stream server stop\n");
    traceclose();
}
/* str2str -------------------------------------------------------------------*/
int main(int argc, char **argv)
{
    int i,opts[]={10000,3000,1000,32768,10},strs[17]={0},nout=0,trlevel=1;
    int stat[17]={0},byte[17]={0},bps[17]={0};
    char *p,*paths[17],str[17][MAXSTRPATH]={{0}};
    char *trfile="",msg[MAXSTRMSG]="";
    
    for (i=0;i<17;i++) paths[i]=str[i];
    
    for (i=1;i<argc;i++) {
        if (!strcmp(argv[i],"-in")&&i+1<argc) {
            if (!(p=strstr(argv[++i],"://"))) {
                fprintf(stderr,"input steam path error : %s\n",argv[i]);
                return -1;
            }
            if (!strncmp(argv[i],"serial",6)) {
                strs[0]=STR_SERIAL; strcpy(paths[0],p+3);
            }
            else if (!strncmp(argv[i],"tcpsvr",6)) {
                strs[0]=STR_TCPSVR; strcpy(paths[0],p+3);
            }
            else if (!strncmp(argv[i],"tcpcli",6)) {
                strs[0]=STR_TCPCLI; strcpy(paths[0],p+3);
            }
            else if (!strncmp(argv[i],"ntrip",5)) {
                strs[0]=STR_NTRIPCLI; strcpy(paths[0],p+3);
            }
            else if (!strncmp(argv[i],"file",4)) {
                strs[0]=STR_FILE; strcpy(paths[0],p+3);
            }
            else {
                fprintf(stderr,"input stream path error : %s\n",argv[i]);
                return -1;
            }
        }
        else if (!strcmp(argv[i],"-out")&&i+1<argc&&nout<16) {
            if (!(p=strstr(argv[++i],"://"))) {
                fprintf(stderr,"output steam path error : %s\n",argv[i]);
                return -1;
            }
            if (!strncmp(argv[i],"serial",6)) {
                strs[1+nout]=STR_SERIAL; strcpy(paths[1+nout++],p+3);
            }
            else if (!strncmp(argv[i],"tcpsvr",6)) {
                strs[1+nout]=STR_TCPSVR; strcpy(paths[1+nout++],p+3);
            }
            else if (!strncmp(argv[i],"tcpcli",6)) {
                strs[1+nout]=STR_TCPCLI; strcpy(paths[1+nout++],p+3);
            }
            else if (!strncmp(argv[i],"ntrip",5)) {
                strs[1+nout]=STR_NTRIPSVR; strcpy(paths[1+nout++],p+3);
            }
            else if (!strncmp(argv[i],"file",4)) {
                strs[1+nout]=STR_FILE; strcpy(paths[1+nout++],p+3);
            }
            else {
                fprintf(stderr,"input stream path error : %s\n",argv[i]);
                return -1;
            }
        }
        else if (!strcmp(argv[i],"-t")&&i+1<argc) trfile=argv[++i];
        else if (!strcmp(argv[i],"-l")&&i+1<argc) trlevel=atoi(argv[++i]);
        else if (*argv[i]=='-') printhelp();
    }
    if (strs[0]==0||nout==0) {
        fprintf(stderr,"specify input stream and output stream(s)\n");
        return -1;
    }
    if (*trfile) {
        traceopen(trfile); tracelevel(trlevel);
    }
printf("str0=%d %s\n",strs[0],paths[0]);
printf("str1=%d %s\n",strs[1],paths[1]);
    signal(SIGINT,stopsvr);
    
    strsvrinit(&strsvr,nout);
    
    fprintf(stderr,"[str2str] stream server start\n");
    strsvrstart(&strsvr,opts,strs,paths,NULL);
    
    while (!sleep(1)) { /* sleep until signal catched */
        strsvrstat(&strsvr,stat,byte,bps,msg);
        fprintf(stderr,"[str2str] ");
        for (i=0;i<nout+1;i++) {
            fprintf(stderr,"STR%d=%d %8d %5d ",i,stat[i],byte[i],bps[i]);
        }
        fprintf(stderr,": %s\r",msg);
    }
    return 0;
}
