/*------------------------------------------------------------------------------
* rtksvr.c : rtk positioning server functions
*
*          Copyright (C) 2007-2009 by T.TAKASU, All rights reserved.
*
* version : $Revision:$ $Date:$
* history : 2009/01/07  1.0 new
*-----------------------------------------------------------------------------*/
#include "rtklib.h"

static const char rcsid[]="$Id:$";

/* write solution header to output stream ------------------------------------*/
static void writesolhead(int format, stream_t *stream)
{
    unsigned char buff[1024];
    int n;
    
    setsolformat(format);
    n=outsolheads(buff);
    strwrite(stream,buff,n);
}
/* write solution to output stream -------------------------------------------*/
static void writesol(gtime_t time, rtk_t *rtk, int stat, obs_t *obs,
                     int *format, stream_t *stream, int nstr)
{
    double rr[6],Qr[9],az[MAXOBS],el[MAXOBS];
    unsigned char buff[1024];
    int i,j,n,ns,sat[MAXOBS],snr[MAXOBS],vsat[MAXOBS];
    
    for (i=0;i<3;i++) {
        rr[i]=stat==1?rtk->xa[i]:rtk->x[i];
        rr[i+3]=rtk->rr[0][i+3];
        for (j=0;j<3;j++) {
            Qr[i+j*3]=stat==1?rtk->Pa[i+j*rtk->na]:rtk->P[i+j*rtk->nx];
        }
    }
    for (i=0;i<nstr;i++) {
        /* 1Hz for nmea */
        if (format[i]==SOLF_NMEA&&DTTOL<time.sec&&time.sec<=1.0-DTTOL) continue;
        
        setsolformat(format[i]);
        n=outsols(buff,time,rr,Qr,rtk->rr[1],stat,rtk->ns);
        strwrite(stream+i,buff,n);
        
        for (j=0;j<obs->n;j++) {
            sat[j]=obs->data[j].sat;
            az[j]=rtk->az[0][SAT(sat[j])-1];
            el[j]=rtk->el[0][SAT(sat[j])-1];
            snr[j]=obs->data[j].SNR[0];
            vsat[j]=rtk->vs[0][SAT(sat[j])-1];
        }
        n=outsolexs(buff,stat,obs->n,sat,az,el,snr,vsat);
        strwrite(stream+i,buff,n);
    }
}
/* decode raw data -----------------------------------------------------------*/
static int decoderaw(rtksvr_t *svr, int i)
{
    nav_t *nav;
    sbsmsg_t sbs={0};
    unsigned char *p,*q,*r;
    int j,stat,fobs=0;
    
    rtksvrlock(svr);
    
    for (p=svr->buff[i],r=p+svr->nb[i];;p=q) {
        
        nav=(svr->navsel==2||svr->navsel==i)?&svr->nav:NULL;
        
        /* decode raw data */
        q=decodelog(svr->raw+i,p,r-p,svr->format[i]+1,0,svr->obs+i,
                    nav,&sbs,&stat);
        
        if (stat==1) { /* obs data */
            fobs=1;
            for (j=0;j<svr->obs[i].n;j++) svr->obs[i].data[j].rcv=i+1;
            sortobs(svr->obs+i);
            svr->nmsg[i][0]++;
        }
        else if (stat== 3) { /* sbas message */
            if (!svr->sbssat||sbs.sat==svr->sbssat) {
                if (svr->nsbs<MAXSBSMSG) {
                    svr->sbsmsg[svr->nsbs++]=sbs;
                }
                else {
                    for (j=0;j<MAXSBSMSG-1;j++) svr->sbsmsg[j]=svr->sbsmsg[j+1];
                    svr->sbsmsg[j]=sbs;
                }
                sbsupdatestat(&sbs,&svr->sbsstat);
            }
            svr->nmsg[i][3]++;
        }
        else if (stat== 2) svr->nmsg[i][1]++; /* ephemeris */
        else if (stat== 9) svr->nmsg[i][2]++; /* ion/utc param */
        else if (stat==-1) svr->nmsg[i][8]++; /* error message */
        
        if (!q||q>=r) { /* no more message */
            svr->nb[i]=0; break;
        }
        if (q==p) { /* imcomplete message */
            if (r<svr->buff[i]+svr->buffsize) {
                if (svr->buff[i]<p) {
                    memcpy(svr->buff[i],p,r-p); svr->nb[i]=r-p;
                }
            }
            else {
                svr->nb[i]=0; /* buffer overflow */
                svr->nmsg[i][9]++;
                tracet(1,"decoderaw: buffer overflow\n");
            }
            break;
        }
    }
    rtksvrunlock(svr);
    
    return fobs;
}
/* rtk server thread ---------------------------------------------------------*/
#ifdef WIN32
static DWORD WINAPI rtksvrthread(void *arg)
#else
static void *rtksvrthread(void *arg)
#endif
{
    rtksvr_t *svr=(rtksvr_t *)arg;
    obs_t obs;
    obsd_t data[MAXOBS*2];
    gtime_t time={0};
    unsigned int tick;
    unsigned char *p,*q;
    int i,j,n,nobs,stat,fobs[2],cycle;
    
    tracet(3,"rtksvrthread:\n");
    
    svr->state=1; obs.data=data;
    svr->tick=tickget();
    
    for (cycle=0;svr->state;cycle++) {
        tick=tickget();
        
        for (i=0;i<2;i++) {
            p=svr->buff[i]+svr->nb[i];
            q=svr->buff[i]+svr->buffsize;
            
            /* read raw data from input stream */
            if ((n=strread(svr->stream+i,p,q-p))<=0) continue;
            
            /* write raw data to log stream */
            strwrite(svr->stream+i+4,p,n);
            svr->nb[i]+=n;
        }
        for (i=0;i<2;i++) { /* decode raw data */
            fobs[i]=decoderaw(svr,i);
        }
        if (fobs[0]) { /* rover observation data received */
            obs.n=0;
            for (i=0;i<svr->obs[0].n;i++) obs.data[obs.n++]=svr->obs[0].data[i];
            for (i=0;i<svr->obs[1].n;i++) obs.data[obs.n++]=svr->obs[1].data[i];
            
            /* rtk positioning */
            rtksvrlock(svr);
            svr->rtkstat=rtkpos(&svr->rtk,obs.data,obs.n,&svr->nav,&svr->sbsstat);
            rtksvrunlock(svr);
            
            /* write solution to output stream */
            if (svr->rtkstat!=SOLQ_NONE) {
                writesol(svr->rtk.time[0],&svr->rtk,svr->rtkstat,svr->obs,
                         svr->format+2,svr->stream+2,2);
            }
        }
        if (svr->rtkstat==SOLQ_NONE&&cycle%(1000/svr->cycle)==0) {
            writesol(time,&svr->rtk,SOLQ_NONE,svr->obs,svr->format+2,
                     svr->stream+2,2);
        }
        /* sleep until next cycle */
        sleepms(svr->cycle-(int)(tickget()-tick));
    }
    for (i=0;i<MAXSTRRTK;i++) strclose(svr->stream+i);
    for (i=0;i<2;i++) free(svr->buff[i]);
    
    return 0;
}
/* initialize rtk server -------------------------------------------------------
* initialize rtk server
* args   : rtksvr_t *svr    IO rtk server
* return : status (0:error,1:ok)
*-----------------------------------------------------------------------------*/
extern int rtksvrinit(rtksvr_t *svr)
{
    rtksvr_t svr0={0};
    eph_t eph0={0};
    int i,j;
    
    tracet(3,"rtksvrinit:\n");
    
    *svr=svr0;
    
    if (!(svr->nav.eph=(eph_t *)malloc(sizeof(eph_t)*MAXSAT))) return 0;
    for (j=0;j<MAXSAT;j++) svr->nav.eph[j]=eph0;
    svr->nav.n=MAXSAT;
    
    for (i=0;i<2;i++) {
        if (!(svr->obs[i].data=(obsd_t *)malloc(sizeof(obsd_t)*MAXOBS))) {
            for (i=0;i<2;i++) free(svr->obs[i].data);
            free(svr->nav.eph);
            return 0;
        }
    }
    for (i=0;i<MAXSTRRTK;i++) strinit(svr->stream+i);
    
    initlock(&svr->lock);
    
    return 1;
}
/* lock/unlock rtk server ------------------------------------------------------
* lock/unlock rtk server
* args   : rtksvr_t *svr    IO rtk server
* return : status (1:ok 0:error)
*-----------------------------------------------------------------------------*/
extern void rtksvrlock(rtksvr_t *svr) {lock(&svr->lock);}
extern void rtksvrunlock(rtksvr_t *svr) {unlock(&svr->lock);}

/* start rtk server ------------------------------------------------------------
* start rtk server thread
* args   : rtksvr_t *svr    IO rtk server
*          int     cycle    I  server cycle (ms)
*          int     buffsize I  input buffer size (bytes)
*          int     *strs    I  input/output/log stream types (STR_???)
*                              types[0]=input stream rover
*                              types[1]=input stream base station
*                              types[2]=output stream solution 1
*                              types[3]=output stream solution 2
*                              types[4]=log stream rover
*                              types[5]=log stream base station
*          char    *paths   I  input/output/log stream paths
*          int     *format  I  input/output/log stream formats (RCVLOG_???)
*          int     navsel   I  navigation message select (0:rover,1:base,2:both)
*          int     sbssat   I  sbas satellite select (0:all,other:prn)
*          char    **cmds   I  input stream start commands
*                              cmds[0]=input stream rover
*                              cmds[1]=input stream base station
*          prcopt_t *prcopt I  rtk processing options
*          solopt_t *solopt I  solution options
* return : status (1:ok 0:error)
*-----------------------------------------------------------------------------*/
extern int rtksvrstart(rtksvr_t *svr, int cycle, int buffsize, int *strs,
                       char **paths, int *formats, int navsel, int sbssat,
                       char **cmds, prcopt_t *prcopt, solopt_t *solopt)
{
    raw_t raw0={{0}};
    char cmdend[]="\r\n";
    int i,j,rw;
    
    tracet(3,"rtksvrstart:\n");
    
    if (svr->state) return 0;
    
    strinitcom();
    svr->cycle=cycle;
    svr->buffsize=buffsize;
    for (i=0;i<4;i++) svr->format[i]=formats[i];
    svr->navsel=navsel;
    svr->sbssat=sbssat;
    svr->nsbs=0;
    svr->rtkstat=0;
    rtkfree(&svr->rtk);
    rtkinit(&svr->rtk,prcopt);
    
    for (i=0;i<2;i++) {
        svr->nb[i]=0;
        if (!(svr->buff[i]=(unsigned char *)malloc(buffsize))) {
            for (i--;i>=0;i--) free(svr->buff[i]);
            return 0;
        }
        for (j=0;j<10;j++) svr->nmsg[i][j]=0;
        svr->raw[i]=raw0;
        svr->obs[i].n=0;
    }
    /* set solution options */
    setsolopt(solopt);
    
    /* open streams */
    for (i=0;i<MAXSTRRTK;i++) {
        rw=i<2?STR_MODE_R:STR_MODE_W;
        if (strs[i]!=STR_FILE) rw|=STR_MODE_W;
        if (strs[i]==STR_FILE) rw|=STR_MODE_T;
        if (stropen(svr->stream+i,strs[i],rw,paths[i])) continue;
        for (i--;i>=0;i--) strclose(svr->stream+i);
        return 0;
    }
    /* write start commands to input streams */
    for (i=0;i<2;i++) {
        if (!cmds[i]) continue;
        strwrite(svr->stream+i,(unsigned char *)cmds[i],strlen(cmds[i]));
        strwrite(svr->stream+i,(unsigned char *)cmdend,2);
    }
    /* write solution header to solution streams */
    for (i=2;i<4;i++) {
        writesolhead(svr->format[i],svr->stream+i);
    }
    /* create rtk server thread */
#ifdef WIN32
    if (!(svr->thread=CreateThread(NULL,0,rtksvrthread,svr,0,NULL))) {
#else
    if (pthread_create(&svr->thread,rtksvrthread,svr))) {
#endif
        for (i=0;i<MAXSTRRTK;i++) strclose(svr->stream+i);
        return 0;
    }
    return 1;
}
/* stop rtk server -------------------------------------------------------------
* start rtk server thread
* args   : rtksvr_t *svr    IO rtk server
*          char    **cmds   I  input stream stop commands
*                              cmds[0]=input stream rover
*                              cmds[1]=input stream base station
* return : none
*-----------------------------------------------------------------------------*/
extern void rtksvrstop(rtksvr_t *svr, char **cmds)
{
    char cmdend[]="\r\n";
    int i;
    
    tracet(3,"rtksvrstop:\n");
    
    /* write stop commands to input streams */
    rtksvrlock(svr);
    for (i=0;i<2;i++) {
        if (!cmds[i]) continue;
        strwrite(svr->stream+i,(unsigned char *)cmds[i],strlen(cmds[i]));
        strwrite(svr->stream+i,(unsigned char *)cmdend,2);
    }
    rtksvrunlock(svr);
    
    /* stop rtk server */
    svr->state=0;
    
    /* free rtk server thread */
#ifdef WIN32
    WaitForSingleObject(svr->thread,10000);
    CloseHandle(svr->thread);
#else
    pthread_join(&svr->thread,NULL);
#endif
}
/* get rtk solution ------------------------------------------------------------
* get current rtk solution
* args   : rtksvr_t *svr    I  rtk server
*          gtime_t *time    O  time of rover
*          double  *rr      O  rover position/velocity (ecef) (m,m/s) (6x1)
*          double  *Qr      O  rover position covariance (3x3)
*          double  *rb      O  base station position (ecef) (m) (3x1)
*          int     *ns      O  number of valid satellites
*          double  *age     O  age of differential (s)
*          double  *ratio   O  ar validationratio-test
* return : status (0:inactive,1:fix,2:float,4:dgps,5:single)
*-----------------------------------------------------------------------------*/
extern int rtksvrgetsol(rtksvr_t *svr, gtime_t *time, double *rr, double *Qr,
                        double *rb, int *ns, double *age, double *ratio)
{
    int i,j,stat;
    
    tracet(4,"rtksvrgetsol:\n");
    
    if (!svr->state) return 0;
    rtksvrlock(svr);
    stat=svr->rtkstat;
    *time=svr->rtk.time[0];
    for (i=0;i<3;i++) {
        rr[i]=stat==1?svr->rtk.xa[i]:svr->rtk.x[i];
        rr[i+3]=svr->rtk.rr[0][i+3];
        rb[i]=svr->rtk.rr[1][i];
    }
    for (i=0;i<3;i++) for (j=0;j<3;j++) {
        Qr[i+j*3]=stat==1?svr->rtk.Pa[i+j*svr->rtk.na]:svr->rtk.P[i+j*svr->rtk.nx];
    }
    *ns=svr->rtk.ns;
    *age=svr->rtk.age;
    *ratio=svr->rtk.ambratio;
    rtksvrunlock(svr);
    return stat;
}
/* get observation data status -------------------------------------------------
* get current observation data status
* args   : rtksvr_t *svr    I  rtk server
*          int     rcv      I  receiver (0:rover,1:base station)
*          gtime_t *time    O  time of observation data
*          int     *sat     O  satellite prn numbers
*          double  *az      O  satellite azimuth angles (rad)
*          double  *el      O  satellite elevation angles (rad)
*          int     *snr1    O  satellite L1 snr (dBHz)
*          int     *snr2    O  satellite L2 snr (dBHz)
*          int     *vsat    O  valid satellite flag
* return : number of satellites
*-----------------------------------------------------------------------------*/
extern int rtksvrostat(rtksvr_t *svr, int rcv, gtime_t *time, int *sat,
                       double *az, double *el, int *snr1, int *snr2, int *vsat)
{
    int i,ns;
    
    tracet(4,"rtksvrostat:\n");
    
    if (!svr->state) return 0;
    rtksvrlock(svr);
    ns=svr->obs[rcv].n;
    if (ns>0) {
        *time=svr->obs[rcv].data[0].time;
    }
    for (i=0;i<ns;i++) {
        sat[i]=svr->obs[rcv].data[i].sat;
        az[i]=svr->rtk.az[0][SAT(sat[i])-1];
        el[i]=svr->rtk.el[0][SAT(sat[i])-1];
        snr1[i]=svr->obs[rcv].data[i].SNR[0];
        snr2[i]=svr->obs[rcv].data[i].SNR[1];
        if (svr->rtk.opt.mode>PMODE_SINGLE) {
            vsat[i]=svr->rtk.vsat[SAT(sat[i])-1][0];
        }
        else {
            vsat[i]=svr->rtk.vs[0][SAT(sat[i])-1];
        }
    }
    rtksvrunlock(svr);
    return ns;
}
/* get stream status -----------------------------------------------------------
* get current stream status
* args   : rtksvr_t *svr    I  rtk server
*          int     *sstat   O  status of streams
*          char    *msg     O  status messages
* return : none
*-----------------------------------------------------------------------------*/
extern void rtksvrsstat(rtksvr_t *svr, int *sstat, char *msg)
{
    int i;
    char s[MAXSTRMSG],*p=msg;
    
    tracet(4,"rtksvrsstat:\n");
    
    rtksvrlock(svr);
    for (i=0;i<MAXSTRRTK;i++) {
        sstat[i]=strstat(svr->stream+i,s);
        if (*s) p+=sprintf(p,"(%d) %s ",i+1,s);
    }
    rtksvrunlock(svr);
}
